import React, {
  useState,
  useCallback,
  useEffect,
} from 'react';
import ReactDOM from 'react-dom';
import {
  csv,
  scaleLinear,
  max,
  format,
  extent,
  range,
  randomUniform,
  randomInt,
  transition,
  easeSin,
} from 'd3';

const width = 600;
const height = 400;
const margin = { top: 50, right: 30, bottom: 50, left: 80 };
const radius = 10;

const generateRandom = (items, minVal, maxVal) => {
  const coords = range(items).map((item) => ({
    x: randomUniform(minVal, maxVal)(),
    y: randomUniform(minVal, maxVal)(),
  }));

  return coords;
};

const App = () => {
  const items = 10;
  const minVal = 0;
  const maxVal = 200;

  const totalDuration = 1000;

  const t = transition('transCoords')
    .duration(totalDuration)
    .ease(easeSin);

  const [data, setData] = useState(
    generateRandom(items, minVal, maxVal)
  );

  const innerHeight = height - margin.top - margin.bottom;
  const innerWidth = width - margin.left - margin.right;

  const onClickFun = (items, minVal, maxVal) => {
    setData(generateRandom(items, minVal, maxVal));
    console.log(data);
  };

  const xVal = (d) => d.x;
  const yVal = (d) => d.y;

  const xScale = scaleLinear()
    .domain(extent(data, xVal))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yVal))
    .range([innerHeight, 0])
    .nice();

  useEffect(() => {
    d3.select('#inner')
      .selectAll('circle')
      .data(data)
      .join(
        (enter) =>
          enter
            .append('circle')
            .attr('r', 10)
            .attr('fill', '#90A4AE')
            .call((enter) =>
              enter
                .attr('cx', (d) => xScale(xVal(d)))
                .attr('cy', (d) => yScale(yVal(d)))
                .attr('fill', 'orange')
            ),
        (update) =>
          update.call((update) => {
            update
              .transition('transCoords')
              .duration(totalDuration)
              .ease(easeSin)
              .attr('cx', (d) => xScale(xVal(d)))
              .attr('cy', (d) => yScale(yVal(d)));
            update
              .transition('HalfCoords1')
              .duration(totalDuration / 2)
              .attr('r', 20)
              .transition('HalfCoords2')
              .duration(totalDuration / 2)
              .attr('r', 10);
            update
              .transition('Fill')
              .duration(totalDuration / 2)
              .attr('fill', 'red')
              .transition('Fill2')
              .duration(totalDuration / 2)
              .attr('fill', 'orange');
          }),
        (exit) => exit.remove()
      );
  }, [data]);

  const AxisBottom = () =>
    xScale.ticks().map((tickValue) => (
      <g
        className="ticks"
        transform={`translate(${xScale(tickValue)},0)`}
      >
        <line y2={innerHeight} />
         <text
          key={tickValue}
          style={{ textAnchor: 'middle' }}
          y={innerHeight + 10}
          dy=".71em"
        >
           {tickValue}
        </text>
      </g>
    ));

  const AxisLeft = () =>
    yScale.ticks().map((tickValue) => (
      <g
        className="ticks"
        transform={`translate(0,${yScale(tickValue)})`}
      >
        <line x2={innerWidth} />
        <text
          key={tickValue}
          style={{ textAnchor: 'end' }}
          x={-10}
          dy=".32em"
        >
          {tickValue}
        </text>
      </g>
    ));

  return (
    <>
      <button
        type="button"
        onClick={() => onClickFun(items, minVal, maxVal)}
      >
        change data
      </button>
      <svg id="chart" width={width} height={height}>
        <g
          transform={`translate(${margin.left}, ${margin.top})`}
          id="inner"
        >
          <rect
            id="background"
            width={innerWidth}
            height={innerHeight}
          />
          <AxisBottom />
          <AxisLeft />
        </g>
      </svg>
    </>
  );
};

const rootElement = document.getElementById('root');
ReactDOM.render(<App />, rootElement);
